/**
 * @file           SDBBOS.h
 * @author         D025086 SAP AG NW DT MaxDB Interfaces\n
 * @brief          SDBBOS is a DBMAPI support library that enables applications to execute 
 * Loader and DBM Server commands
 *
 * This library follows the progamming conventions of the Loader and DBM API for Perl an Python.\n
 * For details about processing of Loader commands see:\n
 * http://dev.mysql.com/doc/maxdb/en/2d/c6c63522b8057ce10000009b38f889/frameset.htm\n
 * For details about processing of DBM Server command see:\n
 * http://dev.mysql.com/doc/maxdb/en/14/774939e5523668e10000000a114084/frameset.htm\n
 * 
\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2001-2005 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end

\endif
*/
#ifndef SDBBOS
#define SDBBOS

/** 
 * Checks if the returned DBM or Loader command was sucessfull.
 */
#define SDBBOS_RESULT_OK(p) (p&&(('O'==(char)p[0])&&('K'==(char)p[1]))||('0'==(char)p[0]))

/** Name of the SDB BOS DBMAPI support library . */
#define SDBBOS_SH_LIB_NAME  "libSDBBOS" 
/** Name of the loader connect function. */
#define SDBBOS_FN_LOADERCONNECT  "LoaderConnect" 
/** Name of the loader execute command function. */
#define SDBBOS_FN_LOADERCMD      "LoaderCmd"
/** Name of the loader release function. */
#define SDBBOS_FN_LOADERRELEASE  "LoaderRelease"
/** Name of the database manager connect function. */
#define SDBBOS_FN_DBMCONNECT     "DBMConnect"
/** Name of the database manager execute command function. */
#define SDBBOS_FN_DBMCMD         "DBMCmd"
/** Name of the database manager release function. */
#define SDBBOS_FN_DBMRELEASE     "DBMRelease"

/**
 * @brief Opens a Loader session.
 * 
 * This function connects the client application with a Loader. To establish a connection 
 * to a specific database, the application must call LoaderCmd and pass a loader cmd using
 * the USE USER command syntax.
 *
 * @param session    [out] A reference to a Loader session.
 * @param error_text [out] A pointer to a buffer where a description is
 *                         available in case of errors.
 * @param error_len  [in]  Length of error_text buffer in bytes.
}
 * @return Zero on success or a none 0 value if the connection to the 
 * Loader could not established. On a none 0 value application may
 * check the error_text for details.
 */
extern "C" int LoaderConnect(void        **session,
                             unsigned char *error_text, 
                             unsigned int   error_len);

/** 
 * Prototype for the LoaderConnect function reference. Used if this library is explicite loaded.
 */
typedef int (*pLoaderConnect)(void        **session,
                             unsigned char *error_text, 
                             unsigned int   error_len);

/**
 * @brief Executes a Loader command.
 *
 * This function executes the specified loader command. It sends the command 
 * to server and receives the answer and data from the reply to the provided 
 * <var>data</var> buffer.
 * 
 * @param session    [in]  A reference to a previously opened Loader session.
 * @param cmd        [in]  A loader command string.
 * @param cmd_len    [in]  The length of the cmd_string buffer.
 * @param data       [out] A buffer to where the returned result buffer of 
 * @param data_size  [in]  The length of data buffer in bytes.
 * @param data_len   [out] The number of bytes returned in the data buffer.
 * @param error_text [out] A pointer to a buffer where a description is
 *                         available in case of errors.
 * @param error_len  [in]  Length of error_text buffer in bytes.
 *
 * @return Zero on success or a none zero value if the command could not be 
 * executed (eg communication errors). On failure the application may check 
 * the error_text for details. On success application must check the data
 * buffer about the result of the command. The data buffer starts with OK 
 * on success.
 */

extern "C" int LoaderCmd(const void *session, 
                         const char *cmd, 
                         unsigned int  cmd_len, 
                         void *data, 
                         unsigned int data_size, 
                         unsigned int *data_len,
                         unsigned char *error_text, 
                         unsigned int error_len);

/**
 * Prototype for the LoaderCmd function reference. Used if this library is explicite loaded.
 */
typedef int (*pLoaderCmd)(const void *session, 
                          const char *cmd, 
                          unsigned int  cmd_len, 
                          void *data, 
                          unsigned int data_size, 
                          unsigned int *data_len,
                          unsigned char *error_text, 
                          unsigned int error_len);

/**
 * @brief Disconnects from the Loader.
 *
 * This function disconnects the client application from a Loader.
 * 
 * @param session    [in]  A reference to a previously opened Loader session.
 *
 * @return none
 */
extern "C" void LoaderRelease(const void *session);

/**
 * Prototype for the LoaderRelease function reference. Used if this library is explicite loaded.
 */
typedef int (*pLoaderRelease)(const void *session);

/**
 * @brief Opens a DBM Server session.
 *
 * This function connects the client application with a DBM Server.
 *
 * @param session         [out] A reference to a DBM Server session.
 *
 * @param database_server [in]  A zero terminated string with the network name. If this string is empty or NULL 
 *                              the local DBM Server will be used. If this buffer contains a hostname or IP address
 *                              the default DBM Server on the specified system is used.
 *
 * @param database_name   [in]  A zero terminated string with the database name. If this string is not NULL 
 *                              or empty a database depended DBM Server will be used.
 * @param dependent_path  [in]  A zero terminated string with the path of the DBM Server. 
 *                              If this string is not NULL or empty the DBM Server at the specified path  
 *                              be used. If there is a database_name specified, this parameter will be
 *                              ignored.
 * @param user_info       [in]  A zero terminated string in the format \<username\>,\<password\>. The specified
 *                              operating system user logs on to the DBM Server. If this string is NULL or empty
 *                              it will be ignored.
 * @param error_text      [out] A pointer to a buffer where a description is available in case of errors.
 * @param error_len       [in]  Length of error_text buffer in bytes.
 *
 * @return Zero on success or a none zero value if the connection to the DBM Server could not established. 
 * On failure the application must check the error_text for details.
 */
extern "C" int DBMConnect(const void   **session,
                          const char    *database_server, 
                          const char    *database_name,
                          const char    *user_info,
                          const char    *dependent_path,
                          unsigned char *error_text, 
                          unsigned int   error_len);

/**
 * Prototype for the DBMConnect function reference. Used if this library is explicite loaded.
 */
typedef int (*pDBMConnect)(const void   **session,
                           const char    *database_server, 
                           const char    *database_name,
                           const char    *user_info,
                           const char    *dependent_path,
                           unsigned char *error_text, 
                           unsigned int   error_len);

/**
 * @brief Executes a DBM Server command.
 *
 * This function executes the specified DBM Server command. It sends the command 
 * to server and receives the answer and data from the reply to the provided 
 * <var>data</var> buffer.
 * 
 * @param session    [in]  A reference to a previously opened DBM Server session.
 * @param cmd        [in]  A DBM Server command string.
 * @param cmd_len    [in]  The length of the cmd_string buffer.
 * @param data       [out] A buffer to where the returned result buffer of 
 * @param data_size  [in]  The length of data buffer in bytes.
 * @param data_len   [out] The number of bytes returned in the data buffer.
 * @param error_text [out] A pointer to a buffer where a description is available in case of errors.
 * @param error_len       [in]  Length of error_text buffer in bytes.
 *
 * @return Zero on success or a none zero value if the command could not be 
 * executed (eg. communication errors). On failure the application must check 
 * the error_text for details. On success application must check the data
 * buffer about the result of the command. The data buffer starts with OK 
 * on success.
 */
extern "C" int DBMCmd(const void    *session, 
                      const char    *cmd, 
                      unsigned int   cmd_len, 
                      void          *data,
                      unsigned int   data_size,
                      unsigned int  *data_len,
                      unsigned char *error_text, 
                      unsigned int   error_len);

/**
 * Prototype for the DBMCmd function reference. Used if this library is explicite loaded.
 */
typedef int (*pDBMCmd)(const void    *session, 
                       const char    *cmd, 
                       unsigned int   cmd_len, 
                       void          *data,
                       unsigned int   data_size,
                       unsigned int  *data_len,
                       unsigned char *error_text, 
                       unsigned int   error_len);


/**
 * @brief Disconnects from the DBM Server.
 *
 * This function disconnects the client application from a DBM Server.
 * 
 * @param session    [in]  A reference to a previously opened DBM Server session.
 *
 * @return none
 */
extern "C" void DBMRelease(const void *session);

/**
 * Prototype for the DBMRelease function reference. Used if this library is explicite loaded.
 */
typedef int (*pDBMRelease)(const void *session);

#endif
/** \mainpage MaxDB DBMAPI support library.
    SDBBOS is a support library that enables applications to execute Loader an DBM Server commands
    <a href="SDBBOS_8h.html">More...</a>.\n
 */
